@@ -1,5 +1,5 @@
 // systemtap translator/driver
-// Copyright (C) 2005-2007 Red Hat Inc.
+// Copyright (C) 2005-2008 Red Hat Inc.
 // Copyright (C) 2005 IBM Corp.
 // Copyright (C) 2006 Intel Corporation.
 //
@@ -19,22 +19,26 @@
 #include "cache.h"
 #include "util.h"
 #include "coveragedb.h"
+#include "git_version.h"
 
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <cerrno>
 #include <cstdlib>
+#include <limits.h>
 
 extern "C" {
 #include <glob.h>
 #include <unistd.h>
+#include <signal.h>
 #include <sys/utsname.h>
 #include <sys/times.h>
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <elfutils/libdwfl.h>
+#include <getopt.h>
 }
 
 using namespace std;
@@ -46,8 +50,8 @@ version ()
   clog
     << "SystemTap translator/driver "
     << "(version " << VERSION << "/" << dwfl_version (NULL)
-    << " built " << DATE << ")" << endl
-    << "Copyright (C) 2005-2007 Red Hat, Inc. and others" << endl
+    << " " << GIT_MESSAGE << ")" << endl
+    << "Copyright (C) 2005-2008 Red Hat, Inc. and others" << endl
     << "This is free software; see the source for copying conditions." << endl;
 }
 
@@ -63,9 +67,11 @@ usage (systemtap_session& s, int exitcode)
     << endl
     << "   or: stap [options] -e SCRIPT    Run given script."
     << endl
+    << "   or: stap [options] -l PROBE     List matching probes."
+    << endl
     << endl
     << "Options:" << endl
-    << "   --         no more options after this" << endl
+    << "   --         end of translator options, script options follow" << endl
     << "   -v         increase verbosity [" << s.verbose << "]" << endl
     << "   -h         show help" << endl
     << "   -V         show version" << endl
@@ -102,8 +108,15 @@ usage (systemtap_session& s, int exitcode)
     << "   -x PID     sets target() to PID" << endl
     << "   -t         collect probe timing information" << endl
 #ifdef HAVE_LIBSQLITE3
-    << "   -q         generate information on tapset coverage"
+    << "   -q         generate information on tapset coverage" << endl
 #endif /* HAVE_LIBSQLITE3 */
+    << "   --kelf     make do with symbol table from vmlinux" << endl
+    << "   --kmap[=FILE]" << endl
+    << "              make do with symbol table from nm listing" << endl
+    << "   --ignore-vmlinux" << endl
+    << "              for testing, pretend vmlinux can't be found" << endl
+    << "   --ignore-dwarf" << endl
+    << "              for testing, pretend vmlinux and modules lack debug info"
     << endl
     ;
   // -d: dump safety-related external references
@@ -115,76 +128,163 @@ usage (systemtap_session& s, int exitcode)
 static void
 printscript(systemtap_session& s, ostream& o)
 {
-  if (s.embeds.size() > 0)
-    o << "# global embedded code" << endl;
-  for (unsigned i=0; i<s.embeds.size(); i++)
+  if (s.listing_mode)
     {
-      embeddedcode* ec = s.embeds[i];
-      ec->print (o);
-      o << endl;
-    }
+      // We go through some heroic measures to produce clean output.
+      set<string> seen;
 
-  if (s.globals.size() > 0)
-    o << "# globals" << endl;
-  for (unsigned i=0; i<s.globals.size(); i++)
-    {
-      vardecl* v = s.globals[i];
-      v->printsig (o);
-      if (s.verbose && v->init)
+      for (unsigned i=0; i<s.probes.size(); i++)
         {
-          o << " = ";
-          v->init->print(o);
+          derived_probe* p = s.probes[i];
+          // NB: p->basest() is not so interesting;
+          // p->almost_basest() doesn't quite work, so ...
+          vector<probe*> chain;
+          p->collect_derivation_chain (chain);
+          probe* second = (chain.size()>1) ? chain[chain.size()-2] : chain[0];
+
+          #if 0
+          cerr << "\tchain[" << chain.size() << "]:" << endl;
+          for (unsigned i=0; i<chain.size(); i++)
+            { cerr << "\t"; chain[i]->printsig(cerr); cerr << endl; }
+          #endif
+
+          stringstream tmps;
+          const probe_alias *a = second->get_alias();
+          if (a)
+            {
+              assert (a->alias_names.size() >= 1);
+              a->alias_names[0]->print(tmps); // XXX: [0] is arbitrary; perhaps print all
+            }            
+          else
+            {
+              assert (second->locations.size() >= 1);
+              second->locations[0]->print(tmps); // XXX: [0] is less arbitrary here, but still ...
+            }
+          string pp = tmps.str();
+
+          // Now duplicate-eliminate.  An alias may have expanded to
+          // several actual derived probe points, but we only want to
+          // print the alias head name once.
+          if (seen.find (pp) == seen.end())
+            {
+              o << pp << endl;
+              seen.insert (pp);
+            }
         }
-      o << endl;
     }
-
-  if (s.functions.size() > 0)
-    o << "# functions" << endl;
-  for (unsigned i=0; i<s.functions.size(); i++)
+  else
     {
-      functiondecl* f = s.functions[i];
-      f->printsig (o);
-      o << endl;
-      if (f->locals.size() > 0)
-	o << "  # locals" << endl;
-      for (unsigned j=0; j<f->locals.size(); j++)
+      if (s.embeds.size() > 0)
+        o << "# global embedded code" << endl;
+      for (unsigned i=0; i<s.embeds.size(); i++)
         {
-	  vardecl* v = f->locals[j];
-	  o << "  ";
-	  v->printsig (o);
-	  o << endl;
-	}
-      if (s.verbose)
+          embeddedcode* ec = s.embeds[i];
+          ec->print (o);
+          o << endl;
+        }
+      
+      if (s.globals.size() > 0)
+        o << "# globals" << endl;
+      for (unsigned i=0; i<s.globals.size(); i++)
         {
-	  f->body->print (o);
-	  o << endl;
-	}
-    }
-
-  if (s.probes.size() > 0)
-    o << "# probes" << endl;
-  for (unsigned i=0; i<s.probes.size(); i++)
-    {
-      derived_probe* p = s.probes[i];
-      p->printsig (o);
-      o << endl;
-      if (p->locals.size() > 0)
-        o << "  # locals" << endl;
-      for (unsigned j=0; j<p->locals.size(); j++)
+          vardecl* v = s.globals[i];
+          v->printsig (o);
+          if (s.verbose && v->init)
+            {
+              o << " = ";
+              v->init->print(o);
+            }
+          o << endl;
+        }
+      
+      if (s.functions.size() > 0)
+        o << "# functions" << endl;
+      for (unsigned i=0; i<s.functions.size(); i++)
         {
-	  vardecl* v = p->locals[j];
-	  o << "  ";
-	  v->printsig (o);
-	  o << endl;
-	}
-      if (s.verbose)
+          functiondecl* f = s.functions[i];
+          f->printsig (o);
+          o << endl;
+          if (f->locals.size() > 0)
+            o << "  # locals" << endl;
+          for (unsigned j=0; j<f->locals.size(); j++)
+            {
+              vardecl* v = f->locals[j];
+              o << "  ";
+              v->printsig (o);
+              o << endl;
+            }
+          if (s.verbose)
+            {
+              f->body->print (o);
+              o << endl;
+            }
+        }
+      
+      if (s.probes.size() > 0)
+        o << "# probes" << endl;
+      for (unsigned i=0; i<s.probes.size(); i++)
         {
-	  p->body->print (o);
-	  o << endl;
-	}
+          derived_probe* p = s.probes[i];
+          p->printsig (o);
+          o << endl;
+          if (p->locals.size() > 0)
+            o << "  # locals" << endl;
+          for (unsigned j=0; j<p->locals.size(); j++)
+            {
+              vardecl* v = p->locals[j];
+              o << "  ";
+              v->printsig (o);
+              o << endl;
+            }
+          if (s.verbose)
+            {
+              p->body->print (o);
+              o << endl;
+            }
+        }
     }
 }
 
+
+int pending_interrupts;
+
+extern "C"
+void handle_interrupt (int /* sig */) 
+{
+  pending_interrupts ++;
+  if (pending_interrupts > 1) // XXX: should be configurable? time-based?
+    {
+      char msg[] = "Too many interrupts received, exiting.\n";
+      int rc = write (2, msg, sizeof(msg)-1);
+      if (rc) {/* Do nothing; we don't care if our last gasp went out. */ ;}
+      _exit (1);
+    }
+}
+
+
+void
+setup_signals (sighandler_t handler)
+{
+  struct sigaction sa;
+
+  sa.sa_handler = handler;
+  sigemptyset (&sa.sa_mask);
+  if (handler != SIG_IGN)
+    {
+      sigaddset (&sa.sa_mask, SIGHUP);
+      sigaddset (&sa.sa_mask, SIGPIPE);
+      sigaddset (&sa.sa_mask, SIGINT);
+      sigaddset (&sa.sa_mask, SIGTERM);
+    }
+  sa.sa_flags = 0;
+
+  sigaction (SIGHUP, &sa, NULL);
+  sigaction (SIGPIPE, &sa, NULL);
+  sigaction (SIGINT, &sa, NULL);
+  sigaction (SIGTERM, &sa, NULL);
+}
+
+
 int
 main (int argc, char * const argv [])
 {
@@ -206,6 +306,7 @@ main (int argc, char * const argv [])
   s.bulk_mode = false;
   s.unoptimized = false;
   s.suppress_warnings = false;
+  s.listing_mode = false;
 
 #ifdef ENABLE_PROLOGUES
   s.prologue_searching = true;
@@ -226,6 +327,9 @@ main (int argc, char * const argv [])
   s.use_cache = true;
   s.tapset_compile_coverage = false;
   s.need_uprobes = false;
+  s.consult_symtab = false;
+  s.ignore_vmlinux = false;
+  s.ignore_dwarf = false;
 
   const char* s_p = getenv ("SYSTEMTAP_TAPSET");
   if (s_p != NULL)  
@@ -276,8 +380,21 @@ main (int argc, char * const argv [])
 
   while (true)
     {
+      int long_opt;
+#define LONG_OPT_KELF 1
+#define LONG_OPT_KMAP 2
+#define LONG_OPT_IGNORE_VMLINUX 3
+#define LONG_OPT_IGNORE_DWARF 4
       // NB: also see find_hash(), usage(), switch stmt below, stap.1 man page
-      int grc = getopt (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:uqw");
+      static struct option long_options[] = {
+        { "kelf", 0, &long_opt, LONG_OPT_KELF },
+        { "kmap", 2, &long_opt, LONG_OPT_KMAP },
+        { "ignore-vmlinux", 0, &long_opt, LONG_OPT_IGNORE_VMLINUX },
+        { "ignore-dwarf", 0, &long_opt, LONG_OPT_IGNORE_DWARF },
+        { NULL, 0, NULL, 0 }
+      };
+      int grc = getopt_long (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:uqwl:",
+                                                          long_options, NULL);
       if (grc < 0)
         break;
       switch (grc)
@@ -303,6 +420,11 @@ main (int argc, char * const argv [])
 	  break;
 
         case 'p':
+          if (s.listing_mode)
+            {
+              cerr << "Listing (-l) mode implies pass 2." << endl;
+              usage (s, 1);
+            }
           s.last_pass = atoi (optarg);
           if (s.last_pass < 1 || s.last_pass > 5)
             {
@@ -377,7 +499,6 @@ main (int argc, char * const argv [])
 	      }
 	  }
 
-	  cerr << "Warning: using '-m' disables cache support." << endl;
 	  s.use_cache = false;
           break;
 
@@ -408,9 +529,9 @@ main (int argc, char * const argv [])
 
         case 's':
           s.buffer_size = atoi (optarg);
-          if (s.buffer_size < 1 || s.buffer_size > 64)
+          if (s.buffer_size < 1 || s.buffer_size > 4095)
             {
-              cerr << "Invalid buffer size (should be 1-64)." << endl;
+              cerr << "Invalid buffer size (should be 1-4095)." << endl;
 	      usage (s, 1);
             }
           break;
@@ -435,6 +556,51 @@ main (int argc, char * const argv [])
           usage (s, 0);
           break;
 
+        case 'l':
+	  s.suppress_warnings = true;
+          s.listing_mode = true;
+          s.last_pass = 2;
+          if (have_script)
+            {
+	      cerr << "Only one script can be given on the command line."
+		   << endl;
+	      usage (s, 1);
+            }
+          cmdline_script = string("probe ") + string(optarg) + " {}";
+          have_script = true;
+          break;
+
+        case 0:
+          switch (long_opt)
+            {
+            case LONG_OPT_KELF:
+	      s.consult_symtab = true;
+	      break;
+            case LONG_OPT_KMAP:
+	      // Leave s.consult_symtab unset for now, to ease error checking.
+              if (!s.kernel_symtab_path.empty())
+		{
+		  cerr << "You can't specify multiple --kmap options." << endl;
+		  usage(s, 1);
+		}
+              if (optarg)
+                s.kernel_symtab_path = optarg;
+              else
+#define PATH_TBD string("__TBD__")
+                s.kernel_symtab_path = PATH_TBD;
+	      break;
+	    case LONG_OPT_IGNORE_VMLINUX:
+	      s.ignore_vmlinux = true;
+	      break;
+	    case LONG_OPT_IGNORE_DWARF:
+	      s.ignore_dwarf = true;
+	      break;
+            default:
+              cerr << "Internal error parsing command arguments." << endl;
+              usage(s, 1);
+            }
+          break;
+
         default:
           usage (s, 1);
           break;
@@ -459,6 +625,18 @@ main (int argc, char * const argv [])
       usage (s, 1);
     }
 
+  if (!s.kernel_symtab_path.empty())
+    {
+      if (s.consult_symtab)
+      {
+        cerr << "You can't specify --kelf and --kmap together." << endl;
+	usage (s, 1);
+      }
+      s.consult_symtab = true;
+      if (s.kernel_symtab_path == PATH_TBD)
+        s.kernel_symtab_path = string("/boot/System.map-") + s.kernel_release;
+    }
+
   if (s.last_pass > 4 && release_changed)
     {
       if (s.verbose)
@@ -484,6 +662,16 @@ main (int argc, char * const argv [])
       usage(s, 1);
     }
 
+  // translate path of runtime to absolute path
+  if (s.runtime_path[0] != '/')
+    {
+      char cwd[PATH_MAX];
+      if (getcwd(cwd, sizeof(cwd)))
+        {
+          s.runtime_path = string(cwd) + "/" + s.runtime_path;
+        }
+    }
+
   int rc = 0;
 
   // override PATH and LC_ALL
@@ -510,7 +698,12 @@ main (int argc, char * const argv [])
 
   // arguments parsed; get down to business
   if (s.verbose > 1)
-    version ();
+    {
+      version ();
+      clog << "Session arch: " << s.architecture
+           << " release: " << s.kernel_release
+           << endl;
+    }
 
   // Create a temporary directory to build within.
   // Be careful with this, as "s.tmpdir" is "rm -rf"'d at the end.
@@ -521,7 +714,9 @@ main (int argc, char * const argv [])
     
     string stapdir = "/stapXXXXXX";
     string tmpdirt = tmpdir_env + stapdir;
+    mode_t mask = umask(0);
     const char* tmpdir = mkdtemp((char *)tmpdirt.c_str());
+    umask(mask);
     if (! tmpdir)
       {
         const char* e = strerror (errno);
@@ -539,6 +734,10 @@ main (int argc, char * const argv [])
   // directory.
   s.translated_source = string(s.tmpdir) + "/" + s.module_name + ".c";
 
+  // Set up our handler to catch routine signals, to allow clean 
+  // and reasonably timely exit.
+  setup_signals(&handle_interrupt);
+
   struct tms tms_before;
   times (& tms_before);
   struct timeval tv_before;
@@ -614,8 +813,10 @@ main (int argc, char * const argv [])
 
           for (unsigned j=0; j<globbuf.gl_pathc; j++)
             {
-              // privilege only for /usr/share/systemtap?
-              
+              if (pending_interrupts)
+                break;
+
+              // XXX: privilege only for /usr/share/systemtap?
               stapfile* f = parser::parse (s, globbuf.gl_pathv[j], true);
               if (f == 0)
                 rc ++;
@@ -681,7 +882,7 @@ main (int argc, char * const argv [])
          << "Try again with more '-v' (verbose) options."
          << endl;
 
-  if (rc || s.last_pass == 1) goto cleanup;
+  if (rc || s.last_pass == 1 || pending_interrupts) goto cleanup;
 
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
@@ -689,7 +890,7 @@ main (int argc, char * const argv [])
   // PASS 2: ELABORATION
   rc = semantic_pass (s);
 
-  if (rc == 0 && s.last_pass == 2)
+  if (s.listing_mode || (rc == 0 && s.last_pass == 2))
     printscript(s, cout);
 
   times (& tms_after);
@@ -731,14 +932,14 @@ main (int argc, char * const argv [])
         {
 	  // If our last pass isn't 5, we're done (since passes 3 and
 	  // 4 just generate what we just pulled out of the cache).
-	  if (s.last_pass < 5) goto cleanup;
+	  if (s.last_pass < 5 || pending_interrupts) goto cleanup;
 
 	  // Short-circuit to pass 5.
 	  goto pass_5;
 	}
     }
 
-  if (rc || s.last_pass == 2) goto cleanup;
+  if (rc || s.listing_mode || s.last_pass == 2 || pending_interrupts) goto cleanup;
 
   // PASS 3: TRANSLATION
 
@@ -767,7 +968,7 @@ main (int argc, char * const argv [])
          << "Try again with more '-v' (verbose) options."
          << endl;
 
-  if (rc || s.last_pass == 3) goto cleanup;
+  if (rc || s.last_pass == 3 || pending_interrupts) goto cleanup;
 
   // PASS 4: COMPILATION
   times (& tms_before);
@@ -775,7 +976,10 @@ main (int argc, char * const argv [])
   rc = compile_pass (s);
 
   if (rc == 0 && s.last_pass == 4)
-    cout << s.hash_path << endl;
+    {
+      cout << ((s.hash_path == "") ? (s.module_name + string(".ko")) : s.hash_path);
+      cout << endl;
+    }
 
   times (& tms_after);
   gettimeofday (&tv_after, NULL);
@@ -797,7 +1001,7 @@ main (int argc, char * const argv [])
 	add_to_cache(s);
 
       // Copy module to the current directory.
-      if (save_module)
+      if (save_module && !pending_interrupts)
         {
 	  string module_src_path = s.tmpdir + "/" + s.module_name + ".ko";
 	  string module_dest_path = s.module_name + ".ko";
@@ -811,7 +1015,7 @@ main (int argc, char * const argv [])
 	}
     }
 
-  if (rc || s.last_pass == 4) goto cleanup;
+  if (rc || s.last_pass == 4 || pending_interrupts) goto cleanup;
 
 
   // PASS 5: RUN
@@ -839,7 +1043,7 @@ pass_5:
  cleanup:
 
   // update the database information
-  if (!rc && s.tapset_compile_coverage) {
+  if (!rc && s.tapset_compile_coverage && !pending_interrupts) {
 #ifdef HAVE_LIBSQLITE3
     update_coverage_db(s);
 #else
@@ -856,6 +1060,10 @@ pass_5:
         clog << "Keeping temporary directory \"" << s.tmpdir << "\"" << endl;
       else
         {
+	  // Ignore signals while we're deleting the temporary directory.
+	  setup_signals (SIG_IGN);
+
+	  // Remove the temporary directory.
           string cleanupcmd = "rm -rf ";
           cleanupcmd += s.tmpdir;
           if (s.verbose>1) clog << "Running " << cleanupcmd << endl;
@@ -865,5 +1073,5 @@ pass_5:
         }
     }
 
-  return rc ? EXIT_FAILURE : EXIT_SUCCESS;
+  return (rc||pending_interrupts) ? EXIT_FAILURE : EXIT_SUCCESS;
 }
