@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * I/O for printing warnings, errors and debug messages
- * Copyright (C) 2005, 2006 Red Hat Inc.
+ * Copyright (C) 2005-2008 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -11,11 +11,6 @@
 #ifndef _IO_C_
 #define _IO_C_
 
-#include "transport/transport.c"
-
-void _stp_print_flush (void);
-void _stp_string_cat_cstr (String str1, const char *str2);
-
 /** @file io.c
  * @brief I/O for printing warnings, errors and debug messages.
  */
@@ -23,23 +18,18 @@ void _stp_string_cat_cstr (String str1, const char *str2);
  * @{
  */
 
-/** private buffer for _stp_log() */
-#define STP_LOG_BUF_LEN 2047
 #define WARN_STRING "WARNING: "
 #define ERR_STRING "ERROR: "
-
-static char _stp_lbuf[NR_CPUS][STP_LOG_BUF_LEN + 1];
-
 enum code { INFO=0, WARN, ERROR, DBUG };
 
 static void _stp_vlog (enum code type, const char *func, int line, const char *fmt, va_list args)
 {
 	int num;
-	char *buf = &_stp_lbuf[get_cpu()][0];
+	char *buf = per_cpu_ptr(Stp_lbuf, get_cpu());
 	int start = 0;
 
 	if (type == DBUG) {
-		start = scnprintf(buf, STP_LOG_BUF_LEN, "\033[36m%s:%d:\033[0m ", func, line);
+		start = _stp_snprintf(buf, STP_LOG_BUF_LEN, "\033[36m%s:%d:\033[0m ", func, line);
 	} else if (type == WARN) {
 		strcpy (buf, WARN_STRING);
 		start = sizeof(WARN_STRING) - 1;
@@ -48,18 +38,18 @@ static void _stp_vlog (enum code type, const char *func, int line, const char *f
 		start = sizeof(ERR_STRING) - 1;
 	}
 
-	num = vscnprintf (buf + start, STP_LOG_BUF_LEN - start, fmt, args);
+	num = _stp_vscnprintf (buf + start, STP_LOG_BUF_LEN - start - 1, fmt, args);
 	if (num + start) {
 		if (buf[num + start - 1] != '\n') {
 			buf[num + start] = '\n';
 			num++;
+			buf[num + start] = '\0';
 		}
-		buf[num + start] = '\0';
-		
+
 		if (type != DBUG)
-			_stp_write(STP_OOB_DATA, buf, start + num + 1);
+			_stp_ctl_write(STP_OOB_DATA, buf, start + num + 1);
 		else {
-			_stp_string_cat_cstr(_stp_stdout,buf);
+			_stp_print(buf);
 			_stp_print_flush();
 		}
 	}
@@ -104,7 +94,7 @@ void _stp_warn (const char *fmt, ...)
  * unload the module and exit. The module will not be 
  * unloaded until after the current probe returns.
  * @note Be careful to not treat this like the Linux exit() 
- * call. You should probably call return immediately after
+ * call. You should probably call return immediately after 
  * calling _stp_exit().
  */
 void _stp_exit (void)
