@@ -9,6 +9,7 @@
  * later version.
  */
 
+#include <linux/uaccess.h>
 #include <linux/types.h>
 #define intptr_t long
 #define uintptr_t unsigned long
@@ -31,9 +32,6 @@
 	       << (sizeof (base) * 8 - (higherbits) - (nbits))))
 
 
-/* These operations are target-specific.  */
-#include <asm/uaccess.h>
-
 /* Given a DWARF register number, fetch its intptr_t (long) value from the
    probe context, or store a new value into the probe context.
 
@@ -75,7 +73,37 @@
     })
 
 
-#if defined __i386__
+#if defined (STAPCONF_X86_UNIREGS) && defined (__i386__)
+
+#define dwarf_register_0(regs)  regs->ax
+#define dwarf_register_1(regs)  regs->cx
+#define dwarf_register_2(regs)  regs->dx
+#define dwarf_register_3(regs)  regs->bx
+#define dwarf_register_4(regs)  ((long) &regs->sp)
+#define dwarf_register_5(regs)  regs->bp
+#define dwarf_register_6(regs)  regs->si
+#define dwarf_register_7(regs)  regs->di
+
+#elif defined (STAPCONF_X86_UNIREGS) && defined (__x86_64__)
+
+#define dwarf_register_0(regs)  regs->ax
+#define dwarf_register_1(regs)  regs->dx
+#define dwarf_register_2(regs)  regs->cx
+#define dwarf_register_3(regs)  regs->bx
+#define dwarf_register_4(regs)  regs->si
+#define dwarf_register_5(regs)  regs->di
+#define dwarf_register_6(regs)  regs->bp
+#define dwarf_register_7(regs)  regs->sp
+#define dwarf_register_8(regs)  regs->r8
+#define dwarf_register_9(regs)  regs->r9
+#define dwarf_register_10(regs) regs->r10
+#define dwarf_register_11(regs) regs->r11
+#define dwarf_register_12(regs) regs->r12
+#define dwarf_register_13(regs) regs->r13
+#define dwarf_register_14(regs) regs->r14
+#define dwarf_register_15(regs) regs->r15
+
+#elif defined __i386__
 
 /* The stack pointer is unlike other registers.  When a trap happens in
    kernel mode, it is not saved in the trap frame (struct pt_regs).
@@ -99,7 +127,7 @@
 #undef fetch_register
 #undef store_register
 
-#define fetch_register(regno)		ia64_fetch_register(regno, c->regs)
+#define fetch_register(regno)		ia64_fetch_register(regno, c->regs, &c->unwaddr)
 #define store_register(regno,value)	ia64_store_register(regno, c->regs, value)
 
 #elif defined __x86_64__
@@ -143,6 +171,61 @@
 
 #endif
 
+
+/* NB: this autoconf is always disabled, pending further performance eval. */
+#if defined STAPCONF_PROBE_KERNEL
+
+/* Kernel 2.6.26 adds probe_kernel_{read,write}, which lets us write
+ * architecture-neutral implementations of kread, kwrite, deref, and
+ * store_deref.
+ *
+ * NB: deref and store_deref shouldn't be used with 64-bit values on 32-bit
+ * platforms, because they will lose data in the conversion to intptr_t.  We
+ * generally want to encourage using kread and kwrite instead.
+ */
+
+#define kread(ptr) ({ \
+        typeof(*(ptr)) _v; \
+        if (probe_kernel_read((void *)&_v, (void *)(ptr), sizeof(*(ptr)))) \
+            DEREF_FAULT(ptr); \
+        _v; \
+    })
+
+#define kwrite(ptr, value) ({ \
+        typeof(*(ptr)) _v; \
+        _v = (typeof(*(ptr)))(value); \
+        if (probe_kernel_write((void *)(ptr), (void *)&_v, sizeof(*(ptr)))) \
+            STORE_DEREF_FAULT(ptr); \
+    })
+
+#define deref(size, addr) ({ \
+    intptr_t _i; \
+    switch (size) { \
+      case 1: _i = kread((u8 *)(addr)); break; \
+      case 2: _i = kread((u16 *)(addr)); break; \
+      case 4: _i = kread((u32 *)(addr)); break; \
+      case 8: _i = kread((u64 *)(addr)); break; \
+      default: __deref_bad(); \
+      /* uninitialized _i should also be caught by -Werror */ \
+    } \
+    _i; \
+  })
+
+#define store_deref(size, addr, value) ({ \
+    switch (size) { \
+      case 1: kwrite((u8 *)(addr), (value)); break; \
+      case 2: kwrite((u16 *)(addr), (value)); break; \
+      case 4: kwrite((u32 *)(addr), (value)); break; \
+      case 8: kwrite((u64 *)(addr), (value)); break; \
+      default: __store_deref_bad(); \
+    } \
+  })
+
+extern void __deref_bad(void);
+extern void __store_deref_bad(void);
+
+#else /* !STAPCONF_PROBE_KERNEL */
+
 #if defined __i386__
 
 #define deref(size, addr)						      \
@@ -584,20 +667,6 @@
 
 #endif /* (s390) || (s390x) */
 
-#define deref_string(dst, addr, maxbytes)				      \
-  ({									      \
-    uintptr_t _addr;							      \
-    size_t _len;							      \
-    unsigned char _c;							      \
-    char *_d = (dst);							      \
-    for (_len = (maxbytes), _addr = (uintptr_t)(addr);			      \
-	 _len > 1 && (_c = deref (1, _addr)) != '\0';			      \
-	 --_len, ++_addr)						      \
-      *_d++ = _c;							      \
-    *_d = '\0';								      \
-    (dst);								      \
-  })
-
 
 #if defined __i386__
 
@@ -630,6 +699,40 @@
 
 #endif
 
+#endif /* STAPCONF_PROBE_KERNEL */
+
+#define deref_string(dst, addr, maxbytes)				      \
+  ({									      \
+    uintptr_t _addr;							      \
+    size_t _len;							      \
+    unsigned char _c;							      \
+    char *_d = (dst);							      \
+    for (_len = (maxbytes), _addr = (uintptr_t)(addr);			      \
+	 _len > 1 && (_c = deref (1, _addr)) != '\0';			      \
+	 --_len, ++_addr)						      \
+      if (_d)								      \
+	 *_d++ = _c;							      \
+    if (_d)								      \
+      *_d = '\0';							      \
+    (dst);								      \
+  })
+
+#define deref_buffer(dst, addr, numbytes)				      \
+  ({									      \
+    uintptr_t _addr;							      \
+    size_t _len;							      \
+    unsigned char _c;							      \
+    char *_d = (dst);							      \
+    for (_len = (numbytes), _addr = (uintptr_t)(addr);			      \
+	 _len >= 1;			                                      \
+	 --_len, ++_addr) {						      \
+      _c = deref (1, _addr);						      \
+      if (_d)								      \
+	 *_d++ = _c;							      \
+    }                                                                         \
+    (dst);								      \
+  })
+
 #define CATCH_DEREF_FAULT()				\
   if (0) {						\
 deref_fault: ;						\
