@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Map Functions
- * Copyright (C) 2005, 2006, 2007 Red Hat Inc.
+ * Copyright (C) 2005, 2006, 2007, 2008 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -15,7 +15,6 @@
  * @brief Implements maps (associative arrays) and lists
  */
 
-#include "alloc.c"
 #include "sym.c"
 #include "stat-common.c"
 #include "map-stat.c"
@@ -138,7 +137,6 @@ int64_t _stp_key_get_int64 (struct map_node *mn, int n)
 
 	if (mn) {
 		res = (*mn->map->get_key)(mn, n, &type).val;
-		dbug("type=%d\n", type);
 		if (type != INT64)
 			res = 0;
 	}
@@ -159,7 +157,6 @@ char *_stp_key_get_str (struct map_node *mn, int n)
 
 	if (mn) {
 		str = (*mn->map->get_key)(mn, n, &type).strp;
-		dbug("type=%d\n", type);
 		if (type != STRING)
 			str = "bad type";
 	}
@@ -205,18 +202,13 @@ static int _stp_map_init(MAP m, unsigned max_entries, int type, int key_size, in
 		
 		for (i = 0; i < max_entries; i++) {
 			if (cpu < 0)
-				tmp = kmalloc(size, STP_ALLOC_FLAGS);
+				tmp = _stp_kmalloc(size);
 			else
-				tmp = kmalloc_node(size, STP_ALLOC_FLAGS, cpu_to_node(cpu));
+				tmp = _stp_kmalloc_node(size, cpu_to_node(cpu));
 		
 			if (!tmp)
 				return -1;
 			
-			if (cpu < 0)
-				_stp_allocated_memory += size;
-			else
-				_stp_allocated_memory += size * num_online_cpus();
-				
 //			dbug ("allocated %lx\n", (long)tmp);
 			list_add((struct list_head *)tmp, &m->pool);
 			((struct map_node *)tmp)->map = m;
@@ -252,10 +244,9 @@ static PMAP _stp_pmap_new(unsigned max_entries, int type, int key_size, int data
 	if (pmap == NULL)
 		return NULL;
 
-	pmap->map = map = (MAP) alloc_percpu (struct map_root);
+	pmap->map = map = (MAP) _stp_alloc_percpu (sizeof(struct map_root));
 	if (map == NULL) 
 		goto err;
-	_stp_allocated_memory += sizeof(struct map_root) * num_online_cpus();
 
 	/* initialize the memory lists first so if allocations fail */
         /* at some point, it is easy to clean up. */
@@ -284,9 +275,9 @@ err1:
 		m = per_cpu_ptr (map, i);
 		__stp_map_del(m);
 	}
-	free_percpu(map);
+	_stp_free_percpu(map);
 err:
-	kfree(pmap);
+	_stp_kfree(pmap);
 	return NULL;
 }
 
@@ -387,13 +378,13 @@ static void __stp_map_del(MAP map)
 	/* free unused pool */
 	list_for_each_safe(p, tmp, &map->pool) {
 		list_del(p);
-		kfree(p);
+		_stp_kfree(p);
 	}
 
 	/* free used list */
 	list_for_each_safe(p, tmp, &map->head) {
 		list_del(p);
-		kfree(p);
+		_stp_kfree(p);
 	}
 }
 
@@ -409,7 +400,7 @@ void _stp_map_del(MAP map)
 
 	__stp_map_del(map);
 
-	kfree(map);
+	_stp_kfree(map);
 }
 
 void _stp_pmap_del(PMAP pmap)
@@ -423,12 +414,12 @@ void _stp_pmap_del(PMAP pmap)
 		MAP m = per_cpu_ptr (pmap->map, i);
 		__stp_map_del(m);
 	}
-	free_percpu(pmap->map);
+	_stp_free_percpu(pmap->map);
 
 	/* free agg map elements */
 	__stp_map_del(&pmap->agg);
 	
-	kfree(pmap);
+	_stp_kfree(pmap);
 }
 
 /* sort keynum values */
@@ -640,136 +631,10 @@ void _stp_map_sortn(MAP map, int n, int keynum, int dir)
 	}
 }
 
-static int print_keytype (char *fmt, int type, key_data *kd)
-{
-	//dbug ("*fmt = %c\n", *fmt);
-	switch (type) {
-	case STRING:
-		if (*fmt != 's')
-			return 1;
-		_stp_print (kd->strp);
-		break;
-	case INT64:
-		if (*fmt == 'x')
-			_stp_printf("%llx", kd->val);
-		else if (*fmt == 'X')
-			_stp_printf("%llX", kd->val);
-		else if (*fmt == 'd')
-			_stp_printf("%lld", kd->val);
-		else if (*fmt == 'p') {
-#if BITS_PER_LONG == 64
-			_stp_printf("%016llx", kd->val);
-#else
-			_stp_printf("%08llx", kd->val);
-#endif
-		} else if (*fmt == 'P')
-			_stp_symbol_print ((unsigned long)kd->val);
-		else
-			return 1;
-		break;
-	default:
-		return 1;
-		break;
-	}
-	return 0;
-}
-
-static void print_valtype (MAP map, char *fmt, struct map_node *ptr)
-{
-	switch (map->type) {
-	case STRING:
-		if (*fmt == 's')
-			_stp_print (_stp_get_str(ptr));
-		break;
-	case INT64:
-	{
-		int64_t val = _stp_get_int64(ptr);
-		if (*fmt == 'x')
-			_stp_printf("%llx", val);
-		else if (*fmt == 'X')
-			_stp_printf("%llX", val);
-		else if (*fmt == 'd')
-			_stp_printf("%lld", val);
-		else if (*fmt == 'p') {
-#if BITS_PER_LONG == 64
-			_stp_printf("%016llx", val);
-#else
-			_stp_printf("%08llx", val);
-#endif
-		} else if (*fmt == 'P')
-			_stp_symbol_print ((unsigned long)val);
-		break;
-	}
-	case STAT:
-	{
-		stat *sd = _stp_get_stat(ptr);
-		_stp_stat_print_valtype (fmt, &map->hist, sd, 0); 
-		break;
-	}
-	default:
-		break;
-	}
-}
-
-/** Print a Map.
- * Print a Map using a format string.
- *
- * @param map Map
- * @param fmt @ref format_string
- */
-void _stp_map_printn (MAP map, int n, const char *fmt)
-{
-	struct map_node *ptr;
-	int type, num;
-	key_data kd;
-	dbug ("print map %lx fmt=%s\n", (long)map, fmt);
-
-	if (n < 0)
-		return;
-
-	foreach (map, ptr) {
-		char *f = (char *)fmt;
-		while (*f) {
-			f = next_fmt (f, &num);
-			if (num) {
-				/* key */
-				kd = (*map->get_key)(ptr, num, &type);
-				if (type != END)
-					print_keytype (f, type, &kd);
-			} else {
-				/* value */
-				print_valtype (map, f, ptr);
-			}
-			if (*f)
-				f++;
-		}
-		_stp_print_char ('\n');
-		if (n && (--n <= 0))
-			break;
-	}
-	_stp_print_char ('\n');
-	_stp_print_flush();
-}
-
-/** Print a Map.
- * Print a Map using a format string.
- *
- * @param map Map
- * @param fmt @ref format_string
- */
-#define _stp_map_print(map,fmt) _stp_map_printn(map,0,fmt)
-
-void _stp_pmap_printn_cpu (PMAP pmap, int n, const char *fmt, int cpu)
-{
-	MAP m = per_cpu_ptr (pmap->map, cpu);
-	_stp_map_printn (m, n, fmt);
-}
-
 static struct map_node *_stp_new_agg(MAP agg, struct hlist_head *ahead, struct map_node *ptr)
 {
 	struct map_node *aptr;
 	/* copy keys and aggregate */
-	dbug("creating new entry in %lx\n", (long)agg);
 	aptr = _new_map_create(agg, ahead);
 	if (aptr == NULL)
 		return NULL;
@@ -922,8 +787,8 @@ MAP _stp_pmap_agg (PMAP pmap)
  */
 #define _stp_pmap_get_agg(pmap) (&pmap->agg)
 
-#define _stp_pmap_printn(map,n,fmt) _stp_map_printn (_stp_pmap_agg(map), n, fmt)
-#define _stp_pmap_print(map,fmt) _stp_map_printn(_stp_pmap_agg(map),0,fmt)
+/* #define _stp_pmap_printn(map,n,fmt) _stp_map_printn (_stp_pmap_agg(map), n, fmt) */
+/* #define _stp_pmap_print(map,fmt) _stp_map_printn(_stp_pmap_agg(map),0,fmt) */
 
 static void _new_map_clear_node (struct map_node *m)
 {
@@ -958,12 +823,10 @@ static struct map_node *_new_map_create (MAP map, struct hlist_head *head)
 			return NULL;
 		}
 		m = (struct map_node *)map->head.next;
-		dbug ("got %lx off head\n", (long)m);
 		hlist_del_init(&m->hnode);
 	} else {
 		m = (struct map_node *)map->pool.next;
 		map->num++;
-		dbug ("got %lx off pool\n", (long)m);
 	}
 	list_move_tail(&m->lnode, &map->head);
 	
