@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * pmap API generator
- * Copyright (C) 2005 Red Hat Inc.
+ * Copyright (C) 2005-2008 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -411,7 +411,9 @@ PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries)
 			m->get_key = KEYSYM(pmap_get_key);
 			m->copy = KEYSYM(pmap_copy_keys);
 			m->cmp = KEYSYM(pmap_key_cmp);
-			m->lock = SPIN_LOCK_UNLOCKED;
+#if NEED_MAP_LOCKS
+			spin_lock_init(m->lock);
+#endif
 		}
 		m = &pmap->agg;
 		m->get_key = KEYSYM(pmap_get_key);
@@ -422,25 +424,19 @@ PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries)
 }
 #else
 /* _stp_pmap_new_key1_key2...val (num, HIST_LINEAR, start, end, interval) */
-/* _stp_pmap_new_key1_key2...val (num, HIST_LOG, buckets) */ 
+/* _stp_pmap_new_key1_key2...val (num, HIST_LOG) */ 
 
 PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries, int htype, ...)
 {
-	int buckets=0, start=0, stop=0, interval=0;
+	int start=0, stop=0, interval=0;
 	PMAP pmap;
-	va_list ap;
 
-	if (htype != HIST_NONE) {
+	if (htype == HIST_LINEAR) {
+		va_list ap;
 		va_start (ap, htype);		
-		if (htype == HIST_LOG) {
-			buckets = va_arg(ap, int);
-			// dbug ("buckets=%d\n", buckets);
-		} else {
-			start = va_arg(ap, int);
-			stop = va_arg(ap, int);
-			interval = va_arg(ap, int);
-			// dbug ("start=%d stop=%d interval=%d\n", start, stop, interval);
-		}
+		start = va_arg(ap, int);
+		stop = va_arg(ap, int);
+		interval = va_arg(ap, int);
 		va_end (ap);
 	}
 
@@ -449,8 +445,7 @@ PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries, int htype, ...)
 		pmap = _stp_pmap_new (max_entries, STAT, sizeof(struct KEYSYM(pmap_node)), 0);
 		break;
 	case HIST_LOG:
-		pmap = _stp_pmap_new_hstat_log (max_entries, sizeof(struct KEYSYM(pmap_node)), 
-					    buckets);
+		pmap = _stp_pmap_new_hstat_log (max_entries, sizeof(struct KEYSYM(pmap_node)));
 		break;
 	case HIST_LINEAR:
 		pmap = _stp_pmap_new_hstat_linear (max_entries, sizeof(struct KEYSYM(pmap_node)),
@@ -469,7 +464,9 @@ PMAP KEYSYM(_stp_pmap_new) (unsigned max_entries, int htype, ...)
 			m->get_key = KEYSYM(pmap_get_key);
 			m->copy = KEYSYM(pmap_copy_keys);
 			m->cmp = KEYSYM(pmap_key_cmp);
-			m->lock = SPIN_LOCK_UNLOCKED;
+#if NEED_MAP_LOCKS
+			spin_lock_init(m->lock);
+#endif
 		}
 		m = &pmap->agg;
 		m->get_key = KEYSYM(pmap_get_key);
@@ -498,7 +495,6 @@ int KEYSYM(__stp_pmap_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 
 	hlist_for_each(e, head) {
 		n = (struct KEYSYM(pmap_node) *)((long)e - sizeof(struct list_head));
-		dbug("map_node =%lx\n", (long)n);
 		if (KEY1_EQ_P(n->key1, key1)
 #if KEY_ARITY > 1
 		    && KEY2_EQ_P(n->key2, key2)
@@ -518,8 +514,6 @@ int KEYSYM(__stp_pmap_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 	}
 
 	/* key not found */
-	dbug("key not found\n");
-
 	n = (struct KEYSYM(pmap_node)*)_new_map_create (map, head);
 	if (n == NULL)
 		return -1;
@@ -531,24 +525,32 @@ int KEYSYM(__stp_pmap_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 int KEYSYM(_stp_pmap_set) (PMAP pmap, ALLKEYSD(key), VSTYPE val)
 {
 	int res;
-	MAP m = per_cpu_ptr (pmap->map, get_cpu());
+	MAP m = per_cpu_ptr (pmap->map, MAP_GET_CPU ());
+#if NEED_MAP_LOCKS
 	if (!spin_trylock(&m->lock))
 		return -3;
+#endif
 	res = KEYSYM(__stp_pmap_set) (m, ALLKEYS(key), val, 0);
+#if NEED_MAP_LOCKS
 	spin_unlock(&m->lock);
-	put_cpu();
+#endif
+        MAP_PUT_CPU ();
 	return res;
 }
 
 int KEYSYM(_stp_pmap_add) (PMAP pmap, ALLKEYSD(key), VSTYPE val)
 {
 	int res;
-	MAP m = per_cpu_ptr (pmap->map, get_cpu());
+	MAP m = per_cpu_ptr (pmap->map, MAP_GET_CPU());
+#if NEED_MAP_LOCKS
 	if (!spin_trylock(&m->lock))
 		return -3;
+#endif
 	res = KEYSYM(__stp_pmap_set) (m, ALLKEYS(key), val, 1);
+#if NEED_MAP_LOCKS
 	spin_unlock(&m->lock);
-	put_cpu();
+#endif
+        MAP_PUT_CPU ();
 	return res;
 }
 
@@ -565,17 +567,17 @@ VALTYPE KEYSYM(_stp_pmap_get_cpu) (PMAP pmap, ALLKEYSD(key))
 	if (pmap == NULL)
 		return NULLRET;
 
-	map = per_cpu_ptr (pmap->map, get_cpu());
+	map = per_cpu_ptr (pmap->map, MAP_GET_CPU ());
 
 	hv = KEYSYM(phash) (ALLKEYS(key));
 	head = &map->hashes[hv];
 
+#if NEED_MAP_LOCKS
 	if (!spin_trylock(&map->lock))
 		return NULLRET;
-
+#endif
 	hlist_for_each(e, head) {
 		n = (struct KEYSYM(pmap_node) *)((long)e - sizeof(struct list_head));
-		dbug("map_node =%lx\n", (long)n);
 		if (KEY1_EQ_P(n->key1, key1)
 #if KEY_ARITY > 1
 		    && KEY2_EQ_P(n->key2, key2)
@@ -591,14 +593,18 @@ VALTYPE KEYSYM(_stp_pmap_get_cpu) (PMAP pmap, ALLKEYSD(key))
 #endif
 			) {
 			res = MAP_GET_VAL((struct map_node *)n);
+#if NEED_MAP_LOCKS
 			spin_unlock(&map->lock);
-			put_cpu();
+#endif
+			MAP_PUT_CPU ();
 			return res;
 		}
 	}
 	/* key not found */
+#if NEED_MAP_LOCKS
 	spin_unlock(&map->lock);
-	put_cpu();
+#endif
+        MAP_PUT_CPU ();
 	return NULLRET;
 }
 
@@ -647,8 +653,10 @@ VALTYPE KEYSYM(_stp_pmap_get) (PMAP pmap, ALLKEYSD(key))
 		map = per_cpu_ptr (pmap->map, cpu);
 		head = &map->hashes[hv];
 
+#if NEED_MAP_LOCKS
 		if (!spin_trylock(&map->lock))
 			return NULLRET;
+#endif
 
 		hlist_for_each(e, head) {
 			n = (struct KEYSYM(pmap_node) *)((long)e - sizeof(struct list_head));
@@ -667,7 +675,6 @@ VALTYPE KEYSYM(_stp_pmap_get) (PMAP pmap, ALLKEYSD(key))
 #endif
 				) {
 				if (anode == NULL) {
-					dbug("agg=%lx ahead=%lx\n", (long)agg, (long)ahead);
 					anode = _stp_new_agg(agg, ahead, (struct map_node *)n);
 				} else {
 					if (clear_agg) {
@@ -678,7 +685,9 @@ VALTYPE KEYSYM(_stp_pmap_get) (PMAP pmap, ALLKEYSD(key))
 				}
 			}
 		}
+#if NEED_MAP_LOCKS
 		spin_unlock(&map->lock);
+#endif
 	}
 	if (anode && !clear_agg) 
 		return MAP_GET_VAL(anode);
@@ -725,19 +734,22 @@ int KEYSYM(__stp_pmap_del) (MAP map, ALLKEYSD(key))
 	}
 
 	/* key not found */
-	dbug("key not found\n");
 	return 0;
 }
 
 int KEYSYM(_stp_pmap_del) (PMAP pmap, ALLKEYSD(key))
 {
 	int res;
-	MAP m = per_cpu_ptr (pmap->map, get_cpu());
+	MAP m = per_cpu_ptr (pmap->map, MAP_GET_CPU ());
+#if NEED_MAP_LOCKS
 	if (!spin_trylock(&m->lock))
 		return -1;
+#endif
 	res = KEYSYM(__stp_pmap_del) (m, ALLKEYS(key));
+#if NEED_MAP_LOCKS
 	spin_unlock(&m->lock);
-	put_cpu();
+#endif
+	MAP_PUT_CPU ();
 	return res;
 }
 
