@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Print Flush Function
- * Copyright (C) 2007 Red Hat Inc.
+ * Copyright (C) 2007-2008 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -15,7 +15,8 @@
  *
  * @note Preemption must be disabled to use this.
  */
-spinlock_t _stp_print_lock = SPIN_LOCK_UNLOCKED;
+
+DEFINE_SPINLOCK(_stp_print_lock);
 
 void EXPORT_FN(stp_print_flush) (_stp_pbuf *pb)
 {
@@ -39,11 +40,13 @@ void EXPORT_FN(stp_print_flush) (_stp_pbuf *pb)
 		else
 			atomic_inc (&_stp_transport_failures);
 #else
-		struct _stp_trace *t = relay_reserve(_stp_utt->rchan, sizeof(*t) + len);
-		if (likely(t)) {
-			t->sequence = _stp_seq_inc();
-			t->pdu_len = len;
-			memcpy((void *) t + sizeof(*t), pb->buf, len);
+		void *buf = relay_reserve(_stp_utt->rchan,
+					sizeof(struct _stp_trace) + len);
+		if (likely(buf)) {
+			struct _stp_trace t = {	.sequence = _stp_seq_inc(),
+						.pdu_len = len};
+			memcpy(buf, &t, sizeof(t)); // prevent unaligned access
+			memcpy(buf + sizeof(t), pb->buf, len);
 		} else 
 			atomic_inc (&_stp_transport_failures);
 #endif
