@@ -1,5 +1,5 @@
 /* main header file
- * Copyright (C) 2005-2007 Red Hat Inc.
+ * Copyright (C) 2005-2008 Red Hat Inc.
  * Copyright (C) 2005, 2006 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -43,21 +43,10 @@
 #define for_each_cpu(cpu)  for_each_cpu_mask((cpu), cpu_possible_map)
 #endif
 
-#ifdef DEBUG
-/** Prints debug line.
- * This function prints a debug message immediately to staprun. 
- * If the last character is not a newline, then one is added. 
- * @param args A variable number of args in a format like printf.
- * @ingroup io
- */
 static void _stp_dbug (const char *func, int line, const char *fmt, ...);
-#define dbug(args...) _stp_dbug(__FUNCTION__, __LINE__, args)
-#define kbug(args...) {printk("%s:%d ",__FUNCTION__, __LINE__); printk(args); }
-#else
-#define dbug(args...) ;
-#define kbug(args...) ;
-#endif /* DEBUG */
-#define errk(args...) {printk("Systemtap Error at %s:%d ",__FUNCTION__, __LINE__); printk(args); }
+void _stp_error (const char *fmt, ...);
+
+#include "debug.h"
 
 /* atomic globals */
 static atomic_t _stp_transport_failures = ATOMIC_INIT (0);
@@ -69,25 +58,32 @@ static struct
 
 #define _stp_seq_inc() (atomic_inc_return(&_stp_seq.seq))
 
-/* TEST_MODE is always defined by systemtap */
-#ifdef TEST_MODE
-#define SYSTEMTAP 1
-#else
-#define MAXTRYLOCK 1000
-#define TRYLOCKDELAY 100
-#endif
-
 #ifndef MAXSTRINGLEN
 #define MAXSTRINGLEN 128
 #endif
 
+#ifndef MAXTRACE
+#define MAXTRACE 20
+#endif
+
+#ifdef CONFIG_FRAME_POINTER
+/* Just because frame pointers are available does not mean we can trust them. */
+#if defined (__i386__) || defined (__arm__)
+#define STP_USE_FRAME_POINTER
+#endif
+#endif
+
+/* dwarf unwinder only tested so far on i386 and x86_64 */
+#if !defined(STP_USE_FRAME_BUFFER) && (defined(__i386__) || defined(__x86_64__))
+#define STP_USE_DWARF_UNWINDER
+#endif
+
 #include "alloc.c"
 #include "print.c"
 #include "string.c"
 #include "io.c"
 #include "arith.c"
 #include "copy.c"
-#include "sym.h"
 #include "sym.c"
 #ifdef STP_PERFMON
 #include "perf.c"
