@@ -1,7 +1,7 @@
 /* -*- linux-c -*- 
  * transport_msgs.h - messages exchanged between module and userspace
  *
- * Copyright (C) Red Hat Inc, 2006-2007
+ * Copyright (C) Red Hat Inc, 2006-2008
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -9,6 +9,8 @@
  * later version.
  */
 
+#define STP_MODULE_NAME_LEN 64
+
 struct _stp_trace {
 	uint32_t sequence;	/* event number */
 	uint32_t pdu_len;	/* length of data after this trace */
@@ -21,37 +23,59 @@ enum
         STP_EXIT,
 	STP_OOB_DATA,
 	STP_SYSTEM,
-	STP_SYMBOLS,
-	STP_MODULE,
 	STP_TRANSPORT,
 	STP_CONNECT,
  	STP_DISCONNECT,
 	STP_BULK,
+	STP_READY,
 #ifdef STP_OLD_TRANSPORT
 	/** deprecated **/
 	STP_BUF_INFO,
 	STP_SUBBUFS_CONSUMED,
 	STP_REALTIME_DATA,
 #endif
+	STP_MAX_CMD
 };
 
+#ifdef DEBUG_TRANS
+static const char *_stp_command_name[] = {
+	"STP_START",
+	"STP_EXIT",
+	"STP_OOB_DATA",
+	"STP_SYSTEM",
+	"STP_TRANSPORT",
+	"STP_CONNECT",
+	"STP_DISCONNECT",
+	"STP_BULK",
+	"STP_READY",
+#ifdef STP_OLD_TRANSPORT
+	"STP_BUF_INFO",
+	"STP_SUBBUFS_CONSUMED",
+	"STP_REALTIME_DATA",
+#endif
+};
+#endif /* DEBUG_TRANS */
+
 /* control channel messages */
 
-/* command to execute: sent to staprun */
+/* command to execute: module->stapio */
 struct _stp_msg_cmd
 {
 	char cmd[128];
 };
 
-/* request for symbol data. sent to staprun */
-struct _stp_msg_symbol
+/* Unwind data. stapio->module */
+struct _stp_msg_unwind
 {
-	int32_t endian;
-	int32_t ptr_size;
+	/* the module name, or "*" for all */
+	char name[STP_MODULE_NAME_LEN];
+	/* length of unwind data */
+	uint32_t unwind_len;
+	/* data ...*/
 };
 
 /* Request to start probes. */
-/* Sent from staprun. Then returned from module. */
+/* stapio->module->stapio */
 struct _stp_msg_start
 {
 	pid_t target;
