@@ -42,8 +42,14 @@
 
 #define MAX_SSOL_SLOTS		1024
 
+#ifdef NO_ACCESS_PROCESS_VM_EXPORT
+static int __access_process_vm(struct task_struct *tsk, unsigned long addr,
+	void *buf, int len, int write);
+#define access_process_vm __access_process_vm
+#else
 extern int access_process_vm(struct task_struct *tsk, unsigned long addr,
 	void *buf, int len, int write);
+#endif
 static int utask_fake_quiesce(struct uprobe_task *utask);
 static void uprobe_release_ssol_vma(struct uprobe_process *uproc);
 
@@ -88,6 +94,15 @@ struct deferred_registration {
 	enum uprobe_type type;
 };
 
+/*
+ * Calling a signal handler cancels single-stepping, so uprobes delays
+ * calling the handler, as necessary, until after single-stepping is completed.
+ */
+struct delayed_signal {
+	struct list_head list;
+	siginfo_t info;
+};
+
 static struct uprobe_task *uprobe_find_utask(struct task_struct *tsk)
 {
 	struct hlist_head *head;
@@ -430,6 +445,7 @@ static int quiesce_all_threads(struct uprobe_process *uproc,
 static void uprobe_free_task(struct uprobe_task *utask)
 {
 	struct deferred_registration *dr, *d;
+	struct delayed_signal *ds, *ds2;
 	struct uretprobe_instance *ri;
 	struct hlist_node *r1, *r2;
 
@@ -439,6 +455,12 @@ static void uprobe_free_task(struct uprobe_task *utask)
 		list_del(&dr->list);
 		kfree(dr);
 	}
+
+	list_for_each_entry_safe(ds, ds2, &utask->delayed_signals, list) {
+		list_del(&ds->list);
+		kfree(ds);
+	}
+
 	hlist_for_each_entry_safe(ri, r1, r2, &utask->uretprobe_instances,
 			hlist) {
 		hlist_del(&ri->hlist);
@@ -551,6 +573,7 @@ static struct uprobe_task *uprobe_add_task(struct task_struct *t,
 	utask->doomed = 0;
 	INIT_HLIST_HEAD(&utask->uretprobe_instances);
 	INIT_LIST_HEAD(&utask->deferred_registrations);
+	INIT_LIST_HEAD(&utask->delayed_signals);
 	INIT_LIST_HEAD(&utask->list);
 	list_add_tail(&utask->list, &uproc->thread_list);
 	uprobe_hash_utask(utask);
@@ -643,6 +666,7 @@ static struct uprobe_process *uprobe_mk_process(struct task_struct *p)
 	uproc->ssol_area.insn_area = NULL;
 	uproc->ssol_area.initialized = 0;
 	mutex_init(&uproc->ssol_area.setup_mutex);
+	/* Initialize rest of area in uprobe_init_ssol(). */
 #ifdef CONFIG_UPROBES_SSOL
 	uproc->sstep_out_of_line = 1;
 #else
@@ -1251,6 +1275,7 @@ static noinline void uprobe_init_ssol(struct uprobe_process *uproc)
 		area->insn_area = ERR_PTR(-ENOMEM);
 		return;
 	}
+	mutex_init(&area->populate_mutex);
 	spin_lock_init(&area->lock);
 	area->next_slot = 0;
 	slot_addr = (char*) area->insn_area;
@@ -1399,8 +1424,10 @@ found_slot:
 	s->last_used = jiffies;
 	s->state = SSOL_ASSIGNED;
 	/* Copy the original instruction to the chosen slot. */
+	mutex_lock(&area->populate_mutex);
 	len = access_process_vm(current, (unsigned long)s->insn,
 					 ppt->insn, MAX_UINSN_BYTES, 1);
+	mutex_unlock(&area->populate_mutex);
         if (unlikely(len < MAX_UINSN_BYTES)) {
 		up_write(&s->rwsem);
 		printk(KERN_ERR "Failed to copy instruction at %#lx"
@@ -1444,12 +1471,18 @@ static struct uprobe_ssol_slot
 static
 struct uprobe_ssol_slot *uprobe_get_insn_slot(struct uprobe_probept *ppt)
 {
-	struct uprobe_ssol_slot *slot = ppt->slot;
+	struct uprobe_ssol_slot *slot;
 
+retry:
+	slot = ppt->slot;
 	if (unlikely(!slot))
 		return uprobe_find_insn_slot(ppt);
 
 	down_read(&slot->rwsem);
+	if (unlikely(slot != ppt->slot)) {
+		up_read(&slot->rwsem);
+		goto retry;
+	}
 	if (unlikely(slot->owner != ppt)) {
 		up_read(&slot->rwsem);
 		return uprobe_find_insn_slot(ppt);
@@ -1560,6 +1593,33 @@ static inline void uprobe_post_ssin(struct uprobe_task *utask,
 
 /* uprobe_pre_ssout() and uprobe_post_ssout() are architecture-specific. */
 
+/*
+ * Delay delivery of the indicated signal until after single-step.
+ * Otherwise single-stepping will be cancelled as part of calling
+ * the signal handler.
+ */
+static u32 uprobe_delay_signal(struct uprobe_task *utask, siginfo_t *info)
+{
+	struct delayed_signal *ds = kmalloc(sizeof(*ds), GFP_USER);
+	if (ds) {
+		ds->info = *info;
+		INIT_LIST_HEAD(&ds->list);
+		list_add_tail(&ds->list, &utask->delayed_signals);
+	}
+	return UTRACE_ACTION_HIDE | UTRACE_SIGNAL_IGN |
+			UTRACE_ACTION_SINGLESTEP | UTRACE_ACTION_NEWSTATE;
+}
+
+static void uprobe_inject_delayed_signals(struct list_head *delayed_signals)
+{
+	struct delayed_signal *ds, *tmp;
+	list_for_each_entry_safe(ds, tmp, delayed_signals, list) {
+		send_sig_info(ds->info.si_signo, &ds->info, current);
+		list_del(&ds->list);
+		kfree(ds);
+	}
+}
+
 /*
  * Signal callback:
  *
@@ -1600,10 +1660,24 @@ static u32 uprobe_report_signal(struct utrace_attached_engine *engine,
 	unsigned long probept;
 	int hit_uretprobe_trampoline = 0;
 	int registrations_deferred = 0;
+	int uproc_freed = 0;
+	struct list_head delayed_signals;
 
-	utask = rcu_dereference((struct uprobe_task *)engine->data);
+	utask = (struct uprobe_task *)rcu_dereference(engine->data);
 	BUG_ON(!utask);
 
+	/*
+	 * info will be null if we're called with action=UTRACE_SIGNAL_HANDLER,
+	 * which means that single-stepping has been disabled so a signal
+	 * handler can be called in the probed process.  That should never
+	 * happen because we intercept and delay handled signals (action =
+	 * UTRACE_ACTION_RESUME) until after we're done single-stepping.
+	 */
+	BUG_ON(!info);
+	if (action == UTRACE_ACTION_RESUME && utask->active_probe &&
+					info->si_signo != SSTEP_SIGNAL)
+		return uprobe_delay_signal(utask, info);
+
 	if (info->si_signo != BREAKPOINT_SIGNAL &&
 					info->si_signo != SSTEP_SIGNAL)
 		goto no_interest;
@@ -1706,13 +1780,21 @@ bkpt_done:
 			uprobe_get_process(uproc);
 		}
 
+		/*
+		 * Delayed signals are a little different.  We want
+		 * them delivered even if all the probes get unregistered
+		 * and uproc and utask go away.  So disconnect the list
+		 * from utask and make it a local list.
+		 */
+		INIT_LIST_HEAD(&delayed_signals);
+		list_splice_init(&utask->delayed_signals, &delayed_signals);
+
 		ret = UTRACE_ACTION_HIDE | UTRACE_SIGNAL_IGN
 			| UTRACE_ACTION_NEWSTATE;
 		utask->state = UPTASK_RUNNING;
 		if (utask->quiescing) {
 			up_read(&uproc->rwsem);
-			if (utask_fake_quiesce(utask) == 1)
-				ret |= UTRACE_ACTION_DETACH;
+			uproc_freed |= utask_fake_quiesce(utask);
 		} else
 			up_read(&uproc->rwsem);
 
@@ -1722,12 +1804,17 @@ bkpt_done:
 			 * we just recycled was the last reason for
 			 * keeping uproc around.
 			 */
-			uprobe_put_process(uproc);
+			uproc_freed |= uprobe_put_process(uproc);
 
 		if (registrations_deferred) {
 			uprobe_run_def_regs(&utask->deferred_registrations);
-			uprobe_put_process(uproc);
+			uproc_freed |= uprobe_put_process(uproc);
 		}
+
+		uprobe_inject_delayed_signals(&delayed_signals);
+
+		if (uproc_freed)
+			ret |= UTRACE_ACTION_DETACH;
 		break;
 	default:
 		goto no_interest;
@@ -1785,7 +1872,7 @@ static u32 uprobe_report_quiesce(struct utrace_attached_engine *engine,
 	struct uprobe_task *utask;
 	struct uprobe_process *uproc;
 
-	utask = rcu_dereference((struct uprobe_task *)engine->data);
+	utask = (struct uprobe_task *)rcu_dereference(engine->data);
 	BUG_ON(!utask);
 	uproc = utask->uproc;
 	if (current == utask->quiesce_master) {
@@ -1886,7 +1973,7 @@ static u32 uprobe_report_exit(struct utrace_attached_engine *engine,
 	struct uprobe_probept *ppt;
 	int utask_quiescing;
 
-	utask = rcu_dereference((struct uprobe_task *)engine->data);
+	utask = (struct uprobe_task *)rcu_dereference(engine->data);
 	uproc = utask->uproc;
 	uprobe_get_process(uproc);
 
@@ -1965,7 +2052,7 @@ static u32 uprobe_report_clone(struct utrace_attached_engine *engine,
 	struct uprobe_process *uproc;
 	struct uprobe_task *ptask, *ctask;
 
-	ptask = rcu_dereference((struct uprobe_task *)engine->data);
+	ptask = (struct uprobe_task *)rcu_dereference(engine->data);
 	uproc = ptask->uproc;
 
 	/*
@@ -2054,7 +2141,7 @@ static u32 uprobe_report_exec(struct utrace_attached_engine *engine,
 	struct uprobe_task *utask;
 	int uproc_freed;
 
-	utask = rcu_dereference((struct uprobe_task *)engine->data);
+	utask = (struct uprobe_task *)rcu_dereference(engine->data);
 	uproc = utask->uproc;
 	uprobe_get_process(uproc);
 
@@ -2267,5 +2354,61 @@ static void zap_uretprobe_instances(struct uprobe *u,
 }
 #endif /* CONFIG_URETPROBES */
 
+#ifdef NO_ACCESS_PROCESS_VM_EXPORT
+/*
+ * Some kernel versions export everything that uprobes.ko needs except
+ * access_process_vm, so we copied and pasted it here.  Fortunately,
+ * everything it calls is exported.
+ */
+#include <linux/pagemap.h>
+#include <asm/cacheflush.h>
+static int __access_process_vm(struct task_struct *tsk, unsigned long addr, void *buf, int len, int write)
+{
+	struct mm_struct *mm;
+	struct vm_area_struct *vma;
+	struct page *page;
+	void *old_buf = buf;
+
+	mm = get_task_mm(tsk);
+	if (!mm)
+		return 0;
+
+	down_read(&mm->mmap_sem);
+	/* ignore errors, just check how much was sucessfully transfered */
+	while (len) {
+		int bytes, ret, offset;
+		void *maddr;
+
+		ret = get_user_pages(tsk, mm, addr, 1,
+				write, 1, &page, &vma);
+		if (ret <= 0)
+			break;
+
+		bytes = len;
+		offset = addr & (PAGE_SIZE-1);
+		if (bytes > PAGE_SIZE-offset)
+			bytes = PAGE_SIZE-offset;
+
+		maddr = kmap(page);
+		if (write) {
+			copy_to_user_page(vma, page, addr,
+					  maddr + offset, buf, bytes);
+			set_page_dirty_lock(page);
+		} else {
+			copy_from_user_page(vma, page, addr,
+					    buf, maddr + offset, bytes);
+		}
+		kunmap(page);
+		page_cache_release(page);
+		len -= bytes;
+		buf += bytes;
+		addr += bytes;
+	}
+	up_read(&mm->mmap_sem);
+	mmput(mm);
+
+	return buf - old_buf;
+}
+#endif
 #include "uprobes_arch.c"
 MODULE_LICENSE("GPL");
