@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2007 Red Hat Inc.
+// Copyright (C) 2005-2008 Red Hat Inc.
 // Copyright (C) 2006 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -222,7 +222,7 @@ struct symbol:
 };
 
 
-struct target_symbol : public expression
+struct target_symbol: public symbol
 {
   enum component_type
     {
@@ -262,8 +262,11 @@ struct functioncall: public expression
 
 struct print_format: public expression
 {
-  bool print_with_format;
   bool print_to_stream;
+  bool print_with_format;
+  bool print_with_delim;
+  bool print_with_newline;
+  bool print_char;
 
   enum format_flag
     {
@@ -284,31 +287,48 @@ struct print_format: public expression
       conv_unsigned_uppercase_hex,
       conv_unsigned_lowercase_hex,
       conv_string,
+      conv_memory,
       conv_literal,
       conv_binary,
       conv_size
     };
 
+  enum width_type
+    {
+      width_unspecified,
+      width_static,
+      width_dynamic
+    };
+
+  enum precision_type
+    {
+      prec_unspecified,
+      prec_static,
+      prec_dynamic
+    };
+
   struct format_component
   {
     unsigned long flags;
     unsigned width;
     unsigned precision;
+    width_type widthtype;
+    precision_type prectype;
     conversion_type type;
     std::string literal_string;
     bool is_empty() const
     {
       return flags == 0
-	&& width == 0
-	&& precision == 0
+	&& widthtype == width_unspecified
+	&& prectype == prec_unspecified
 	&& type == conv_unspecified
 	&& literal_string.empty();
     }
     void clear()
     {
       flags = 0;
-      width = 0;
-      precision = 0;
+      widthtype = width_unspecified;
+      prectype = prec_unspecified;
       type = conv_unspecified;
       literal_string.clear();
     }
@@ -320,11 +340,14 @@ struct print_format: public expression
 
   std::string raw_components;
   std::vector<format_component> components;
+  format_component delimiter;
   std::vector<expression*> args;
   hist_op *hist;
 
   static std::string components_to_string(std::vector<format_component> const & components);
   static std::vector<format_component> string_to_components(std::string const & str);
+  static bool parse_print(const std::string &name, bool &stream, 
+	            bool &format, bool &delim, bool &newline, bool &_char);
 
   void print (std::ostream& o) const;
   void visit (visitor* u);
@@ -411,6 +434,7 @@ struct functiondecl: public symboldecl
 {
   std::vector<vardecl*> formal_args;
   std::vector<vardecl*> locals;
+  std::vector<vardecl*> unused_locals;
   statement* body;
   functiondecl ();
   void print (std::ostream& o) const;
@@ -446,6 +470,8 @@ struct block: public statement
   std::vector<statement*> statements;
   void print (std::ostream& o) const;
   void visit (visitor* u);
+  block () {}
+  block (statement* car, statement* cdr);
 };
 
 
@@ -537,6 +563,7 @@ struct next_statement: public statement
 
 
 struct probe;
+struct derived_probe;
 struct probe_alias;
 struct embeddedcode;
 struct stapfile
@@ -553,8 +580,6 @@ struct stapfile
 };
 
 
-
-
 struct probe_point
 {
   struct component // XXX: sort of a restricted functioncall
@@ -567,9 +592,13 @@ struct probe_point
   std::vector<component*> components;
   const token* tok; // points to first component's functor
   bool optional;
+  bool sufficient;
+  expression* condition;
   void print (std::ostream& o) const;
   probe_point ();
+  probe_point(const probe_point& pp);
   probe_point(std::vector<component*> const & comps,const token * t);
+  std::string str();
 };
 
 std::ostream& operator << (std::ostream& o, const probe_point& k);
@@ -578,12 +607,15 @@ std::ostream& operator << (std::ostream& o, const probe_point& k);
 struct probe
 {
   std::vector<probe_point*> locations;
-  block* body;
+  statement* body;
   const token* tok;
   std::vector<vardecl*> locals;
+  std::vector<vardecl*> unused_locals;
   probe ();
   void print (std::ostream& o) const;
   virtual void printsig (std::ostream &o) const;
+  virtual void collect_derivation_chain (std::vector<probe*> &probes_list);
+  virtual const probe_alias *get_alias () const { return 0; }
   virtual probe* basest () { return this; }
   virtual ~probe() {}
   bool privileged;
@@ -719,6 +751,7 @@ struct varuse_collecting_visitor: public functioncall_traversing_visitor
   void visit_print_format (print_format *e);
   void visit_assignment (assignment *e);
   void visit_arrayindex (arrayindex *e);
+  void visit_target_symbol (target_symbol *e);
   void visit_symbol (symbol *e);
   void visit_pre_crement (pre_crement *e);
   void visit_post_crement (post_crement *e);
@@ -784,6 +817,7 @@ struct deep_copy_visitor: public visitor
 {
   std::stack<void *> targets;
 
+  static expression *deep_copy (expression *s);
   static statement *deep_copy (statement *s);
   static block *deep_copy (block *s);
 
@@ -821,7 +855,7 @@ struct deep_copy_visitor: public visitor
   virtual void visit_hist_op (hist_op* e);
 };
 
-template <typename T> static void
+template <typename T> void
 require (deep_copy_visitor* v, T* dst, T src)
 {
   *dst = NULL;
@@ -834,33 +868,10 @@ require (deep_copy_visitor* v, T* dst, T src)
     }
 }
 
-template <> static void
-require <indexable *> (deep_copy_visitor* v, indexable** dst, indexable* src)
-{
-  if (src != NULL)
-    {
-      symbol *array_src=NULL, *array_dst=NULL;
-      hist_op *hist_src=NULL, *hist_dst=NULL;
+template <> void
+require <indexable *> (deep_copy_visitor* v, indexable** dst, indexable* src);
 
-      classify_indexable(src, array_src, hist_src);
-
-      *dst = NULL;
-
-      if (array_src)
-	{
-	  require <symbol*> (v, &array_dst, array_src);
-	  *dst = array_dst;
-	}
-      else
-	{
-	  require <hist_op*> (v, &hist_dst, hist_src);
-	  *dst = hist_dst;
-	}
-      assert (*dst);
-    }
-}
-
-template <typename T> static void
+template <typename T> void
 provide (deep_copy_visitor* v, T src)
 {
   assert(!v->targets.empty());
