@@ -36,12 +36,19 @@ int main()
   ubuf.actime = 1;
   ubuf.modtime = 1135641600;
   utime("foobar", &ubuf);
-  // utime ("foobar", \[1970/01/01-00:00:01, 2005/12/27-00:00:00\]) = 0
+#ifdef __ia64__
+  // utimes ("foobar", \[1.000000\]\[1135641600.000000\]) =
+#else
+  // utime ("foobar", \[Thu Jan  1 00:00:01 1970, Tue Dec 27 00:00:00 2005\]) = 0
+#endif
 
   ubuf.actime =  1135690000;
   ubuf.modtime = 1135700000;
   utime("foobar", &ubuf);
-  // utime ("foobar", \[2005/12/27-13:26:40, 2005/12/27-16:13:20\]) = 0
-
+#ifdef __ia64__
+  // utimes ("foobar", \[1135690000.000000\]\[1135700000.000000\]) =
+#else
+  // utime ("foobar", \[Tue Dec 27 13:26:40 2005, Tue Dec 27 16:13:20 2005\]) = 0
+#endif
   return 0;
 }
